###########################################
### Data Descriptor: ACDC, Amphibia’s Curated Database of Cytochrome-b sequences
### Citation: van den Burg MP, Herrando-Pérez S, Vieites DR (2020) Data Descriptor: ACDC, Amphibia’s Curated Database of Cytochrome-b sequences. Scientific Data. doi:
### Code by Salvador Herrando Pérez (salherra@gmail.com) on 27/07/2019. Revised on 25/03/2020.
###########################################
NOTE: the scrip has been written for steps 1 to 8 to be run in a sequence in one single R session. This has two implications.
(1) Step 8 will only work if steps 1 to 7 have been run (in that order) before. The same applies to any intermediate step relative to the steps that precede it. 
(2) The script might fail to run if one step is run twice during the same R session. Therefore... 
...if users make a mistake while running any given step, we suggest they troubleshoot what the problem is, then close the active R session and re-run the entire script in a new R session. 

# SCRIPT STEPS
# STEP 1: Install and load packages
# STEP 2: Load packages and set working directory
# STEP 3: Retrieve accession numbers from GenBank
# STEP 4: Remove NC sequences and output genus-ordered fasta file with tidy sequence names
# STEP 5: Convert fasta file to data.frame and output table of species names flagging name typos, if any
# STEP 6: Intraspecific sequence alignments
# STEP 7: Intrageneric sequence alignments and divergence estimates
# STEP 8: Identify information per base pair for database alignment

## SCRIPT OUTPUTS SAVED TO WORKING DIRECTORY / Names of output directories and files given automatically
# OUTPUT 1 from STEP 3 (1 FASTA FILE) / All cytb sequences = OUTPUT1_Amphibia_Cytb_GenBank_RAW.fasta
# OUTPUT 2 from STEP 4 (1 FASTA FILE) / Target cytb sequences = OUTPUT2_Amphibia_Cytb_GenBank_FINAL.fasta
# OUTPUT 3 from STEP 5 (1 TEXT FILE) / Species names with associated number of sequences per species = OUTPUT3_SpeciesNames.txt
# OUTPUT 4 from STEP 6 (2 FASTA FILEs PER SPECIES to new working directory = OUTPUT5_IntraspecificAlignments) / Raw sequences (one of the fasta files with name start = SpeciesRaw_ followed by species name) and pairwise sequence alignments (the other fasta file one of the fasta files with name = SpeciesAligned_ followed by species name) for each species = directory: OUTPUT4_IntraspecificAlignments
# OUTPUT 5 from STEP 7 (2 FASTA FILEs PER GENUS WITH >1 SPECIES to new working directory = OUTPUT5_IntragenericAlignments) / Raw sequences (one of the fasta files with name = GenusRaw_ followed by genus name) and pairwise sequence alignments (the other fasta file one of the fasta files with name = GenusAligned_ followed by genus name) for each genus = directory: OUTPUT5_IntragenericAlignments
# OUTPUT 6 from STEP 7 (1 TEXT FILE PER GENUS WITH >1 SPECIES to new working directory = OUTPUT6_IntragenericDivergences) / Pairwise divergence estimates (fasta files named therein = Divergence_PairwiseValues_ followed by genus name): directory: OUTPUT6_IntragenericDivergences
###############################################################################################################################################################################################################################################
###############################################################################################################################################################################################################################################


###################################################################################################################################
##STEP 1: Install and load packages
###################################################################################################################################

#1.1/INSTALLATION
##Installation and package (names and manuals)
#source("http://www.bioconductor.org/biocLite.R"); biocLite("Biostrings"); biocLite("msa"); biocLite("muscle"); biocLite("phyloseq") #code required prior to 2019
chooseCRANmirror() #choose CRAN repository
install.packages("BiocManager") #install BiocManager - do only once
BiocManager::install(c("Biostrings", "msa", "muscle","phyloseq")) #install packages
#BiocManager::install() #use only to update Bioconductor packages
BiocManager::version() #check version
install.packages("ape")
install.packages("crayon")
install.packages("gdata")
install.packages("plyr")
install.packages("rentrez")
install.packages("rlist")
install.packages("seqinr")
install.packages("seqRFLP")
install.packages("stringr")
install.packages("vegan")

##Access package manuals
browseURL('https://cran.r-project.org/web/packages/BiocManager/vignettes/BiocManager.html') #BiocManager
browseURL('https://bioconductor.org/packages/release/bioc/html/Biostrings.html') #Biostrings
browseURL('https://bioconductor.org/packages/release/bioc/html/msa.html') #msa
browseURL('https://bioconductor.org/packages/release/bioc/html/phyloseq.html') #muscle
browseURL('https://cran.r-project.org/web/packages/ape/index.html') #ape
browseURL('https://cran.r-project.org/web/packages/crayon/index.html') #crayon
browseURL('https://cran.r-project.org/web/packages/gdata/index.html') #gdata
browseURL('https://cran.r-project.org/web/packages/plyr/index.html') #plyr
browseURL('https://cran.r-project.org/web/packages/rentrez/index.html') #rentrez
browseURL('https://cran.r-project.org/web/packages/rlist/index.html') #rlist
browseURL('https://cran.r-project.org/web/packages/seqinr/index.html') #seqinr
browseURL('https://cran.r-project.org/web/packages/seqRFLP/index.html') #seqRFLP
browseURL('https://cran.r-project.org/web/packages/stringr/index.html') #stringr
browseURL('https://cran.r-project.org/web/packages/vegan/index.html') #vegan

##Empty R environment
rm(list = ls())

#1.2/LOADING
##Load packages - notice that some packages should not be loaded here but downstream 
library(ape); library(Biostrings); library(crayon); library(msa); library(phyloseq)
library(plyr); library(rentrez); library(seqinr); library(stringr); library(vegan)
sessionInfo() #double-check target (and dependent) packages loaded correctly
###################################################################################################################################
#NOTE: RUN STEP 1 ONLY THE FIRST TIME YOU ARE USING THIS SCRIPT ON YOUR COMPUTER
###################################################################################################################################


###################################################################################################################################
#STEP 2: Set working directory
###################################################################################################################################

#Assign path to your working directory to object wd_today_day / Set one operative system, whether WINDOWS or MAC
#Example for WINDOWS
#wd_today_name <- "C:/Users/salvador/Desktop/cytb/cytb_March2020"
#Example for MAC
wd_today_name <- "/Users/salvador/Desktop/cytb/cytb_May2020"
#Set path
setwd(wd_today_name); wd_today <- setwd(wd_today_name) #/path for Mac
###################################################################################################################################
#NOTE: LOAD PACKAGES IN STEP 1.2 TO RUN CODE HEREAFTER
cat(bold(blue(paste("Your working directory is ", wd_today_name, sep=""))),"\n"); cat(italic(blue(paste("Load packages in step 1.2 to run code hereafter"))))
###################################################################################################################################


###################################################################################################################################
#STEP 3: Retrieve accession names and sequences from GenBank
###################################################################################################################################

#Create key-word string for GENBANK searches for amphibian cytochrome-b sequences with length range 1-50000 base pairs
entrez_db_searchable(db="nuccore", config = NULL) #searchable fields in database = nuccore 
amphibia_cytb <- "(Amphibia[ORGN]) AND (cytochrome-b) AND (1[SLEN] : 50000[SLEN]) AND (1970/01/01[PDAT] : 2018/02/01[PDAT]) NOT (cf OR sp OR aff OR unisexual OR hybrid OR unverified OR predicted)"; amphibia_cytb; class(amphibia_cytb); length(amphibia_cytb)
# amphibia_cytb <- "(Mantella[ORGN])" #run entire code only with genus = Mantella

#Number of hits (=cytbs) using amphibia_cytb
cytbs <- entrez_search(db="nuccore", term=amphibia_cytb, retmax=100)$count; cytbs
cat(bold(blue(paste("number of hits = ", cytbs, " sequences of amphibian cytb", sep=""))))

#Retrieve accession names
amphibia_cytbIDs <- entrez_search(db="nuccore", term=amphibia_cytb, retmax = cytbs, use_history=TRUE); amphibia_cytbIDs
#number of hits, where hits should be equal to hits
cytb_ids <- amphibia_cytbIDs$count; cytb_ids; cytbs; cat(bold(blue(paste("number of sequence ids = ", cytb_ids, " (number of sequences should be equal to number of sequence ids)", sep=""))))

#Round number of subsets of 1000 sequences to the nearest ten
subsetsN <- ceiling(cytb_ids/1000); subsetsN; subsetsN*1000; cytb_ids
cat(bold(blue(paste("Number of subsets of 1000 sequences each = ", subsetsN, sep=""))))

#Retrieve (and save to working directory) accession names and nucleotidic sequences in subsets of 1000 GenBank records each
cat(bold(blue(paste("Your working directory is ", wd_today_name, sep=""))),"\n"); cat(italic(blue(paste("The following loop will store object OUTPUT1_Amphibia_Cytb_GenBank_RAW.fasta in working directory"))),"\n")
start =1
for (loops in c(1:subsetsN)) {
  cytb_subset <- entrez_fetch(db="nuccore", web_history = amphibia_cytbIDs$web_history, rettype="fasta", retmax=1000, retstart = start)
  if (loops == 1) {write(cytb_subset, "OUTPUT1_Amphibia_Cytb_GenBank_RAW.fasta", sep="\n")}
  if (loops > 1) {write(cytb_subset, "OUTPUT1_Amphibia_Cytb_GenBank_RAW.fasta", sep="\n", append = TRUE)}
  if (loops == 1) {cat(bold(blue(paste("Sequence retrieval in progress", sep=""))),"\n")}
  if (loops == 1) {cat(italic(green(paste("sequences retrieved = ", 1000, sep=""))),"\n")}
  start = start+1000
  if (loops > 1 & loops < subsetsN) {cat(italic(green(paste("sequences retrieved = ", start-1, sep=""))),"\n")}
  if (loops == subsetsN) {cat(bold(blue(paste("Task completed: number of sequences retrieved  = ", cytbs, sep=""))),"\n")}
  if (loops == subsetsN) {cat(bold(blue(paste("Sequences stored in working directory as file = OUTPUT1_Amphibia_Cytb_GenBank_RAW.fasta", sep=""))),"\n")}
}
###################################################################################################################################
#NOTE: OUTPUT FILE IN STEP 3 REQUIRED TO RUN STEP 4
cat(bold(blue(paste("Your working directory is ", wd_today_name, sep=""))),"\n"); cat(italic(blue(paste("Output file in step 3 required to run step 4"))),"\n")
###################################################################################################################################


###################################################################################################################################
#STEP 4: Remove NC sequences and output genus-ordered fasta file with tidy sequence names (Genus_Species_AccessionName)
###################################################################################################################################

##Read fasta file
Amphibia_CYTB_seqinr <- read.fasta(file = "OUTPUT1_Amphibia_Cytb_GenBank_RAW.fasta", seqtype = "DNA", as.string = TRUE, forceDNAtolower = FALSE); cat(bold(blue(paste("File = OUTPUT1_Amphibia_Cytb_GenBank_RAW.fasta read from working directory", sep=""))), "\n"); cat(italic(blue(paste("Number of initial sequences of amphibian cytb = ", length(Amphibia_CYTB_seqinr), sep=""))), "\n")
##Visualize first sequence in Amphibia_CYTB_seqinr
Amphibia_CYTB_seqinr[[1]] 

#Subset accession names
acNumber <- attr(Amphibia_CYTB_seqinr, "names"); acNumber; length(acNumber) #accession names
#Remove characters after . (inclusive)
acNumber <- gsub("\\..*","", acNumber); acNumber <- gsub("^.*\\|", "", acNumber); acNumber #in accession names, gsub converts unwanted characters (prior to first comma) to void ("")

#Subset index of NC dates
seq_NC <- which(grepl("NC_",acNumber) == TRUE); seq_NC; length(seq_NC) #detects position of NC sequences
#Remove NC records from fasta file (OUTPUT 1) if any
if (length(seq_NC) > 0) {
  Amphibia_CYTB_seqinr <- Amphibia_CYTB_seqinr[-c(seq_NC)]; length(Amphibia_CYTB_seqinr); cat(bold(blue("NC_ sequences removed from dataset")),"\n")
  #remove NC labels from vector of accession numbers 
  acNumber <- acNumber[-seq_NC]
  } 
if (length(seq_NC) == 0) {Amphibia_CYTB_seqinr <- Amphibia_CYTB_seqinr; length(Amphibia_CYTB_seqinr); cat(bold(blue("0 NC_ sequences found in dataset")),"\n")}
length(acNumber); acNumber #final subset of sequences

acNumber <- gsub("^.*\\|", "", acNumber); acNumber; class(acNumber); length(acNumber) #in accession numbers, gsub converts unwanted characters (prior to first comma) to void ("")
###################################################################################################################################
#NOTE: CREATION OF OBJECT AmphibiaCytb_out BELOW IS TIME-CONSUMING, PERHAPS IT'S TIME FOR A COFFEE
cat(bold(blue("NEXT STEP = Retrieval of species names for target sequences (AmphibiaCytb_out) is time-consuming, perhaps it's time for a coffee :-)")),"\n")
###################################################################################################################################

#Get genus and species names using labels in acNumber
#Note default options in read.GenBank below store both accesssion numbers (seq.names = access.nb) and species names (species.names = TRUE)
AmphibiaCytb_out <- read.GenBank(acNumber, seq.names = access.n, species.names = TRUE) 
cat(italic(blue(paste("Number of input sequences in OUTPUT 1 = ", length(Amphibia_CYTB_seqinr), " should be the same as the number of record names retrieved in AmphibiaCytb_out = ", length(AmphibiaCytb_out)), sep="")),"\n")

#Keep only genus_species words since many records include subspecies names and other words and characters
sp_names <- matrix(paste(attr(AmphibiaCytb_out, "species"), sep="_")); sp_names #space between taxonomic scientific names converted to _
spNamesMat <- matrix (attr(AmphibiaCytb_out, "species")); spNamesMat #creates matrix (rows = species names) needed for gen_sp below
gen_sp <- apply(spNamesMat, 1, function(x) sub("(_[^_]+)_.*", "\\1", x)); gen_sp #get tidy species names for each row in spNamesMat / assumes first two words are genus_species and remove everything thereafter / output = character vector

#Create vector with notation genus_species_accessionName
myLabel <- apply(matrix(names(AmphibiaCytb_out)), 1, function(x) gsub("_","", x)); myLabel #removes _ from accession numbers
Amphibia_CYTB_seqs_IDs <- paste(gen_sp, myLabel, sep="_"); Amphibia_CYTB_seqs_IDs
table(table(Amphibia_CYTB_seqs_IDs)) #confirm there are no duplicates
length(Amphibia_CYTB_seqs_IDs)

#Rename each sequence
attr(Amphibia_CYTB_seqinr, "names") <- Amphibia_CYTB_seqs_IDs; attr(Amphibia_CYTB_seqinr, "names"); cat(bold(blue("Each target sequence named as Genus_Species_AccessionNumber")),"\n")

#Order fasta file by genus, then by species, then by accession number
Amphibia_CYTB_seqinr <- Amphibia_CYTB_seqinr[order(names(Amphibia_CYTB_seqinr))]; names(Amphibia_CYTB_seqinr); cat(bold(blue("Sequences ordered by genus name, then by species name")),"\n")

#Output genus-ordered fasta file
write.fasta(sequences = Amphibia_CYTB_seqinr, names = names(Amphibia_CYTB_seqinr), file.out = "OUTPUT2_Amphibia_Cytb_GenBank_FINAL.fasta"); cat(bold(blue(paste("File containing final set of target cytb sequences = Amphibia_Cytb_GenBank_Record_certain_output.fasta stored in working directory", sep=""))),"\n")
###################################################################################################################################
#NOTE: RUN STEPS 1 TO 4 BEFORE RUNNING STEP 5
cat(bold(blue(paste("Your working directory is ", wd_today_name, sep=""))),"\n"); cat(italic(blue(paste("Run steps 1 to 4 before running step 5"))),"\n")
###################################################################################################################################


###################################################################################################################################
#STEP 5: Convert fasta file to data.frame and output table of species names flagging name typos, if any
###################################################################################################################################

#Load packages / respect loading order to avoid code malfunction, so re-load the packages below even if some have already been loaded in Step 1.2
library (ape); library(Biostrings)
library(crayon); library(gdata)
library(msa); library(phyloseq) 
library(plyr); library(rentrez)
library(seqinr); library(vegan)
cat(bold(blue(paste("Your working directory is ", wd_today_name, sep=""))),"\n"); cat(italic(blue(paste("Respect package loading order to avoid code malfunction, so re-load the packages below even if some have already been loaded in Step 1.2"))),"\n")


#Load data / fastaSpecies is not = AmphibiaCytbF (see above) in which as.string = FALSE by default
fastaSpecies <- read.fasta(file = "OUTPUT2_Amphibia_Cytb_GenBank_FINAL.fasta", seqtype = "DNA", as.string = TRUE, forceDNAtolower = FALSE); cat(bold(blue(paste("File = OUTPUT2_Amphibia_Cytb_GenBank_FINAL.fasta read from working directory", sep=""))),"\n")
seqN <- length(fastaSpecies); cat(bold(blue(paste("Number of sequences = ", seqN, sep=""))))

#Loop to extract IDs and sequences / Repository with rownumber = number of sequences and colnumber = 2 (seqID and sequence)
rep_fastaSpecies <- matrix(NA,seqN,2); dim(rep_fastaSpecies); cat(bold(blue(paste("Store sequences in dataframe with 1st column for sequence labels and 2nd column for nucleotidic sequences", sep=""))),"\n")
for (i_seq in c(1:seqN)){
  id_seq <- attributes(fastaSpecies[[i_seq]])$name #extracts ID of sequences
  string_seq <- fastaSpecies[[i_seq]][1] #extracts sequence string
  rep_fastaSpecies[i_seq,] <- c(id_seq, string_seq)
}
rep_fastaSpecies <- data.frame(rep_fastaSpecies); dim(rep_fastaSpecies); class(rep_fastaSpecies) #convert matrix to data.frame
names(rep_fastaSpecies); colnames(rep_fastaSpecies) <- c("idName", "Seq"); names(rep_fastaSpecies) #rename columns

#Formatting sequence IDs in two columns including Genus.species (Species) and AccessionNumbers (Info)
split_names <- data.frame(read.table(text = c(matrix(rep_fastaSpecies$idName)), sep = "_", colClasses = "character")); dim(split_names) #split IDnames by character _
names(split_names); colnames(split_names) <- c("Genus", "species", "Info"); names(split_names) #rename columns
gensp_merged <- paste(split_names$Genus, split_names$species, sep="."); length(gensp_merged) #merges genus and species names = Genus.species

#Output dataframe in 3 columns including including Genus.species (Species) and AccessionNumbers (Info) and sequences (Seq)
final_species <- data.frame(gensp_merged, split_names$Info, rep_fastaSpecies$Seq)
names(final_species); colnames(final_species) <- c("Species", "Info", "Seq"); names(final_species) #changes column names
str(final_species); dim(final_species); class(final_species) #summary

#List and table of species names
cat(bold(blue(paste("Store sequences in dataframe with 1st column for Genus.Species labels, 2nd column for AccessionNumbers and 3rd column for nucleotidic sequences", sep=""))),"\n")
final_species$Species #species names
table(final_species$Species) #number of sequences per species (dataframed below)
ouputSpeciesNames <- data.frame(table(final_species$Species)); colnames(ouputSpeciesNames); colnames(ouputSpeciesNames) <- c("SpeciesNames", "SequenceNumber"); colnames(ouputSpeciesNames); dim(ouputSpeciesNames) 

#Save output file in working directory
##List of unique species names
write.table(ouputSpeciesNames, "OUTPUT3_SpeciesNamesUnique.txt",sep="\t",row.names=FALSE); cat(bold(blue(paste("Species names and number of sequences per species stored in working directory as file = OUTPUT3_SpeciesNames", sep=""))),"\n")
##List of species names as in original fasta file (OUTPUT2_Amphibia_Cytb_GenBank_FINAL.fasta) / Use the text file below to modify species names if necessary
write.table(gensp_merged, "OUTPUT3_SpeciesNamesMultiple.txt",sep="\t",row.names=FALSE); cat(bold(blue(paste("Species names and number of sequences per species stored in working directory as file = OUTPUT3_SpeciesNames", sep=""))),"\n")
###################################################################################################################################
#NOTE: STEP 5 MUST BE RUN PRIOR TO RUNNING STEP 6
#NOTE: RUN STEP 6 ONLY ONCE PER R SESSION - IF YOU WANT TO RUN STEP 6 A SECOND TIME, SAVE YOUR WORK, REMOVE OR RENAME FOLDER = OUTPUT4_IntraspecificAlignments IN WORKING DIRECTORY, AND START A NEW R SESSION
cat(bold(blue(paste("Your working directory is ", wd_today_name, sep=""))),"\n"); cat(italic(blue(paste("Step 5 must be run prior to running step 6"))),"\n"); cat(italic(blue(paste("Step 6 must be run only once per R session - if you want to run step 6 again, save your work, remove or rename folder = OUTPUT4_IntraspecificAlignments in working directory, and start a new R session"))),"\n")
###################################################################################################################################


###################################################################################################################################
#STEP 6: Intraspecific sequence alignments
###################################################################################################################################

#Load packages  / respect loading order to avoid code malfunction, so re-load the packages below even if some have already been loaded in steps 1.2 and 5
cat(bold(blue(paste("Your working directory is ", wd_today_name, sep=""))),"\n"); cat(italic(blue(paste("Respect package loading order to avoid code malfunction, so re-load the packages below even if some have already been loaded in step 1.2 and 5"))),"\n")
library(seqRFLP); library(msa); cat(bold(blue(paste("This section aligns sequences for species with >1 sequence", sep=""))),"\n"); cat(italic(blue(paste("Hereafter, intraspecific sequence alignments are based on ClustalW in msa() - please use 'muscle' in external software for finer aligments", sep=""))),"\n")

#Rename data.frame with species names, accession numbers and sequences (3 columns)
species <- final_species
dim(species) #number of rows (=sequences) and columns
names(species) #column headings where Species = species names, Info = accession number, Seq = nucleotidic sequence
class(species) #file type

#Have you renamed species names in "OUTPUT3_SpeciesNamesMultiple.txt" externally? - if so, you can read in the new set of names here which must be in the same order as in OUTPUT2_Amphibia_Cytb_GenBank_FINAL.fasta and OUTPUT3_SpeciesNamesMultiple.txt
#choose one option whether yes or no
#renamed = "yes"
renamed = "no"

#Get vector spN = one number assigned to each species (numeric IDs), with species numeric IDs times their frequency in the data.frame
if(renamed == "yes") {
  speciesRenamed <- read.table("OUTPUT3_SpeciesNamesMultiple.txt", header=T)
  species$Species <- speciesRenamed
    }

#Get vector spN = one number assigned to each species (numeric IDs), with species numeric IDs times their frequency in the data.frame
species$Species #vector with species names across sequences
speNames <- unique(species$Species); speNames; length(speNames)
speciesR <- as.data.frame(table(species$Species))[,2]; speciesR; length(speciesR); length(speNames) #sample size by species ordered alphabetically by genus names, where number of species = length(speciesR) = length(speNames)
sp <- data.frame(c(1:length(speNames)),speNames,speciesR); names(sp) = c("speciesN", "species","frequency"); sp; cat(bold(blue(paste("Find rigth above 3-column table including numerical ID (column = speciesN), species names (column = species), and sample size by species (column = frequency)", sep=""))),"\n")
spN <- rep(c(sp$speciesN), c(sp$frequency)); length (spN); dim(species)[1] #number of sequences = length(spN) = dim(species)[1]

#Get vector labelS2 = index for species with >1 sequence
sp_final1 <- data.frame(spN, species); names(sp_final1); class(sp_final1); dim(sp_final1) #dataframe including sequences (sequences) ordered by species name (species) and vector 'spN' showing number of sequences per species
subset1 <- which(table(spN)>1); subset1; length(subset1) #IDs of species having >1 sequence
cat(bold(blue(paste("The dataset contains a total of ", length(speNames), " species, of which ", length(subset1), " have >1 sequence", sep=""))),"\n")

#Remove species with 1 sequence for intraspecific alignments since 1 sequence has no pair to align against
sp_final2 <- sp_final1[sp_final1$spN %in% c(subset1),]; sp_final2; names(sp_final2); class(sp_final2); dim(sp_final2) 
labelS2 <- unique(sp_final2$spN); labelS2; length(labelS2) #index used in loop below to subset species with >1 sequence
cat(bold(blue(paste("labelS2 contains within-fasta-file position of species with >1 sequence", sep=""))),"\n")

##Users must choose one, and only one, of options C1 to C5
#C1###alignment type = 'method' in msa
#alig <- "Muscle"
alig <- "ClustalW"
#C2###penalty for opening gap = 'gapOpening' in msa()
Openal <- 400 #default in msa, modify as appropriate
#C3###penalty for extention gap = 'gapExtension' in msa()
Epenal <- 0 #default in msa, modify as appropriate
#C4###number of iterations = 'maxiters" in msa()
Niter <- 16 #default in msa and muscle, modify as appropriate
#C5###type of genetic data = 'type' in msa() / Note that this script has been tested only for DNA sequences
nu_ac <- "dna"
#nu_ac <- "rna"
#nu_ac <- "protein"
?msa; cat(bold(blue(paste("Alignment criteria are: ", sep=""))),"\n"); cat((blue(paste("(i) alignment method = ", alig, sep=""))),"\n"); cat((blue(paste("(ii) opening-gap penalty = ", Openal, sep=""))),"\n"); cat((blue(paste("(iii) extension-gap penalty = ", Epenal, sep=""))),"\n"); cat((blue(paste("(iv) number of iterations = ", Niter, sep=""))),"\n"); cat((blue(paste("(v) genetic sequences = ", nu_ac, sep=""))),"\n"); cat(italic(green(paste("The former are the default criteria in msa - modify if appropriate", sep=""))),"\n")

#Loop doing multiple aligments for species with >1 sequence
alignSP <- list() #repository of alignments for species with >1 sequence

#Create directory to store results from loop
newDIRname <- "OUTPUT4_IntraspecificAlignments"; newDIRname #name of new directory to store outputs of loop
cat(bold(blue(paste("Folder = ", newDIRname, " has been created to store intraspecific alignments", sep=""))),"\n")
newDIR <- dir.create(newDIRname); newDIR #creation of new directory
setwd(paste(wd_today_name, "/", newDIRname, sep="")); newDIRname1 <- setwd(paste(wd_today_name, "/", newDIRname, sep="")); newDIRname1; cat(bold(blue(paste("Hereafter, alignment outputs will be stored in folder: ", newDIRname1, sep=""))))

#Loop
for(i in labelS2) {
  #subsetting by i
  species_i <- sp_final2[sp_final2$spN == i,]; species_i; names(species_i) #subset species i
  spName <- species_i$Species[1]; spName #get name of species i
  pre_align <- as.data.frame(species_i[,c("Info","Seq")]); pre_align; dim(pre_align); names(pre_align) #subset columns 3 (AccessionNumber) and 4 (sequences)
  pre_align$Info <- factor(paste(species_i$Species,species_i$Info, sep=".")); pre_align$Info; dim(pre_align) #change sequence IDs from only AccessionNumber to also include SpeciesNames
  
  #msa() alignment / store alignments per species in alignSP at slot i (i=species)
  pre_align.fasta <- dataframe2fas(pre_align, file = paste("SpeciesRaw_", spName, ".fasta", sep = "")) #dataframe2fas (from seqRFLP) converts dataframe to fasta format needed in msa AND automatically outputs fasta file (editable in external software, e.g., BioEdit, Geneious) to working directory, name file = spName
  align_SPi <- msa(pre_align.fasta[seq(2,length(pre_align.fasta),2)], method = alig, order = "input", gapOpening = Openal, gapExtension = Epenal, maxiters = Niter, type = nu_ac); align_SPi #do alignments / order = "input" keeps sequences in the order of input file
  alignSP[[i]] <- list(align_SPi) 
  cat(bold(green(paste("Raw sequences for ", spName, " ready for alignment",sep=""))),"\n"); cat(italic(green(paste("Alignment in progress",sep=""))),"\n")
  #do alignments
  align <- as.data.frame(align_SPi@unmasked); align; class(align); dim(align) #retrieves aligned sequences
  rownames(align)
  rownames(align) <- pre_align$Info; rownames(align) #retrieve Accession Numbers and Species names
  species_iA <- dataframe2fas(align, file = paste("SpeciesAligned_", spName, ".fasta", sep = "")) #dataframe2fas (from seqRFLP) converts dataframe to fasta format needed in msa AND automatically outputs fasta file (editable in external software, e.g., BioEdit, Geneious) to working directory, name file = spName
  cat(italic(silver(paste("Alignment completed and stored in working directory",sep=""))),"\n")
  #NOTE: loop generates two files per species, one including unaligned sequences and named =SpeciesRaw_SpeciesName.fasta, another including aligned sequences and named = SpeciesAligned_SpeciesName.fasta
}
length(alignSP); class(alignSP); cat(bold(blue(paste("Intraspecific alignments completed and stored in folder = ", newDIRname, " within your working directory", sep=""))),"\n"); cat((blue(paste("For a finer intraspecific alignment, please export the fasta files for 'muscle' alignments in a software of your choice", sep=""))),"\n")

######return to initial working directory
setwd(wd_today_name); cat(bold(blue(paste("Your working directory is back to ", wd_today_name, sep=""))))

#######################################################
#VISUALIZATION OF SEQUENCES FROM SPECIES WITH 1 SINGLE SEQUENCE / code will show no result if all species have >1 species
#table showing number of sequences per species
sp
#vector extracting species = 1 sequence from table sp / if none, output = <0 rows> (or 0-length row.names)
table_spSeq <- sp[sp$frequency ==1,]; table_spSeq 
#vector extracting names of species = 1 sequence / if none, output = character(0)
sp1names <- c(as.character(table_spSeq$species)); sp1names
#dataframe extracting sequences for species = 1 sequence / if none, output = <0 rows> (or 0-length row.names)
species[species$Species %in% sp1names,]
#######################################################

###################################################################################################################################
#NOTE: STEP 5 MUST BE RUN PRIOR TO RUNNING STEP 7
#NOTE: YOU CAN RUN STEP 7 ONLY ONCE PER R SESSION - IF YOU WANT TO RUN STEP 7 A SECOND TIME, SAVE YOUR WORK, REMOVE/RENAME...
# ... FOLDERS = OUTPUT5_IntragenericAlignments AND OUTPUT6_IntragenericDivergences IN WORKING DIRECTORY, AND START A NEW R SESSION
cat(bold(blue(paste("Your working directory is ", wd_today_name, sep=""))),"\n"); cat(italic(blue(paste("Step 5 must be run prior to running step 7"))),"\n"); cat(italic(blue(paste("Step 6 must be run only once per R session - if you want to run step 7 again, save your work, remove or rename folders = OUTPUT5_IntragenericAlignments and OUTPUT6_IntragenericDivergences in working directory, and start a new R session"))),"\n")
###################################################################################################################################


###################################################################################################################################
#STEP 7: Intrageneric sequence alignments and divergence estimates
###################################################################################################################################

######set working directory
setwd(wd_today_name); wd_today <- setwd(wd_today_name) #/path for Mac
cat(bold(blue(paste("Your working directory is ", wd_today_name, sep=""))))

#load packages
cat(bold(blue(paste("Your working directory is ", wd_today_name, sep=""))),"\n"); cat(italic(blue(paste("Respect package loading order to avoid code malfunction, so re-load the packages below even if some have already been loaded in Step 1.2"))),"\n")
library(seqRFLP)
library(msa)
library(rlist)

##RUN STEP 5 BEFORE RUNNING CODE HEREAFTER
cat(bold(blue(paste("Align sequences and estimate sequence divergence for genera with >1 species", sep=""))),"\n")

##rename data.frame with species names, accession numbers and sequences (3 columns)
species <- final_species
dim(species) #number of rows (=sequences) and columns
names(species) #column headings where Species = species names, Info = accession number, Seq = nucleotidic sequence
class(species) #file type

##get vector spN = one number assigned to each species (numeric IDs), with species numeric IDs times their frequency in the data.frame
species$Species #vector with species names across sequences
speNames <- unique(species$Species); speNames #vector of species names
speciesR <- as.data.frame(table(species$Species))[,2]; speciesR; length(speciesR); length(speNames) #sample size by species ordered alphabetically by genus names, where number of species = length(speciesR) = length(speNames)
sp <- data.frame(c(1:length(speNames)),speNames,speciesR); names(sp) = c("speciesN", "species","frequency"); sp #3-column dataframe including species numerical ID (speciesN), species names (species), and sample size by species (frequency)
spN <- rep(c(sp$speciesN), c(sp$frequency)); spN; length (spN); dim(species)[1] #number of sequences = length(spN) = dim(species)[1]

##Get species names
#speciesO <- species[order(species$Species),]; speciesO; class(speciesO); dim(speciesO); names(speciesO) #order by species names / spN is ordered so / NOT NEEDED ANY MORE SINCE SEQUENCES ARE ORDERED BY NAME IN PREVIOUS STEP
sp_final3 <- data.frame(spN, species); names(sp_final3); names(sp_final3) <- c("spN","species", "ID","sequences"); names(sp_final3); class(sp_final3); dim(sp_final3) #dataframe including sequences (sequences) ordered by species name (species) and vector 'spN' showing number of sequences per species
dimO <- dim(sp_final3)[1]; dimO # number of sequences with a genus.species name
sp.names <- matrix(sp_final3$species); sp.names #vector with genus.species names

#loop splitting column with 'genus.species' in two columns, first = genus, second = species names
rep.names <- matrix(NA,dimO,2)
for (w in c(1:dimO)) {
  rowgenus <- c(t(data.frame(strsplit(sp.names[w,], ".", fixed = TRUE))))
  rep.names[w,] <- rowgenus
}
rep.names; rep.names <- data.frame(rep.names); class(rep.names); dim(rep.names); rep.names
speciesG <- cbind(rep.names,sp_final3); speciesG #merge split names with original sequence file
names(speciesG) <- c("nameGenus", "nameSpecies", "Species", "Info", "Seq") # renaming columns
numtaxFrame <- cbind(speciesG$nameGenus, speciesG$nameSpecies); numtaxFrame #gives ordered numbers to genus and species
numtax <- numtaxFrame[,1]; numtax #vector with numeric IDs for genus in the same order as in SpeciesG
speciesG <- cbind(numtax, speciesG); speciesG #merge numeric indexes (numtax) with speciesG
names(speciesG) <- c("numGenus", "nameGenus", "nameSpecies", "numberSpecies", "Species", "Info", "Seq") #renaming columns where numGenus contains numeric IDs for genus
speciesG

#subset genus represented by >1 species
tableGS <- table(speciesG$nameGenus, speciesG$nameSpecies)/table(speciesG$nameGenus, speciesG$nameSpecies); tableGS #table counting species presences (columns) per genus (rows)
scoreGS <- apply(tableGS, 1, function(x) sum(x, na.rm = TRUE)); scoreGS #number of species per genus
#genus >1 species
namesG <- names(which(scoreGS>1)); namesG #genus names represented by >1 species
ge_final <- speciesG[speciesG$nameGenus %in% namesG,]; ge_final #subset sequences from genus represented by >1 species
labelG2 <- unique(ge_final$numGenus); labelG2 #index used in loop below to subset genus represented by >1 species
cat(bold(blue(paste("labelG2 contains within-fasta-file position of genera with >1 species", sep=""))),"\n")
#genus = 1 species
names1sp <- names(which(scoreGS==1)); names1sp #genus names represented by >1 species / if none, output = character(0)
ge_final1sp <- speciesG[speciesG$nameGenus %in% names1sp,]; ge_final1sp #subset sequences from genus represented by >1 species / if none, output = <0 rows> (or 0-length row.names)
labelG_1sp <- unique(ge_final1sp$numGenus); labelG_1sp #index used in loop below to subset genus represented by >1 species / if none, output = integer(0)

##Choices: users must choose one, and only 1, of options C1 to C5
#C1###alignment type = 'method' in msa
#alig <- "Muscle"
alig <- "ClustalW"
#C2###penalty for opening gap = 'gapOpening'
Openal <- 400 #default in msa (must be negative), modify as appropriate
#C3###penalty for extention gap = 'gapExtension'
Epenal <- 0 #default in msa (must be negative), modify as appropriate
#C4###number of iterations = 'maxiters" in msa
Niter <- 16 #default in msa and muscle, modify as appropriate
nu_ac <- "dna"
#nu_ac <- "rna"
#nu_ac <- "protein"
?msa; cat(bold(blue(paste("Alignment criteria are: ", sep=""))),"\n"); cat((blue(paste("(i) alignment method = ", alig, sep=""))),"\n"); cat((blue(paste("(ii) opening-gap penalty = ", Openal, sep=""))),"\n"); cat((blue(paste("(iii) extension-gap penalty = ", Epenal, sep=""))),"\n"); cat((blue(paste("(iv) number of iterations = ", Niter, sep=""))),"\n"); cat((blue(paste("(v) genetic sequences = ", nu_ac, sep=""))),"\n"); cat(italic(green(paste("The former are the default criteria in msa - modify if appropriate", sep=""))),"\n")

##Loop doing multiple aligments for species with >1 sequence
#repositories
alignGE <- list() #repository of alignments for species with >1 seq (part1)
divBELOW <- list() #repository of species pairs BELOW divergence threshold (part2)
divABOVE <- list() #repository of species pairs ABOVE divergence threshold (part2)
cat(bold(blue(paste("AlignGE is the repository of alignments for species with >1 seq", sep=""))),"\n")
cat(bold(blue(paste("divBELOW is the repository of species pairs BELOW divergence threshold", sep=""))),"\n")
cat(bold(blue(paste("divBELOW is the repository of species pairs ABOVE divergence threshold", sep=""))),"\n")
#divergence threshold
div.threshold <- 3 #user selects threshold
cat(bold(blue(paste("Selected divergence threshold = ", div.threshold, sep=""))),"\n")

##Create folder to story results from loop
newDIRname <- "OUTPUT5_IntragenericAlignments"; newDIRname #name of new folder to store alignment outputs
newDIR <- dir.create(newDIRname); newDIR #creation of new folder
cat(bold(blue(paste("Folder = ", newDIRname, "has been created to store intrageneric alignments", sep=""))),"\n")
#set folder for alignments
setwd(paste(wd_today_name, "/", newDIRname, sep="")); nowDIR <- getwd(); nowDIR
#name of folder for divergences
newDIRname2 <- "OUTPUT6_IntragenericDivergences"; newDIRname2 #name of new directory to store divergence outputs of loop
cat(bold(blue(paste("Folder = ", newDIRname2, "has been created to store divergence estimates", sep=""))),"\n")
#full path to divergence folder
newDIRname3 <- paste(wd_today_name, "/", newDIRname2, sep=""); newDIRname3; cat(bold(blue(paste("Hereafter, alignment outputs will be stored in folder: ", nowDIR, sep=""))),"\n"); cat(bold(blue(paste("... and divergence outputs will be stored in folder: ", newDIRname3, sep=""))),"\n")

for(i in labelG2){ ## loop for i values from genus
  
  ##PART 1: ALIGNMENTS ##############################
  
  #subsetting by i
  genus_i <- ge_final[ge_final$numGenus == i,]; genus_i; head(genus_i) #subset genus i
  geName <- genus_i$nameGenus[1]; geName #get name of genus i
  pre_align <- as.data.frame(genus_i[,c("Info","Seq")]); pre_align; class(pre_align); dim(pre_align) #subsets columns 5 (Accession number) and 6 (sequences)
  pre_align$Info <- factor(paste(genus_i$Species,pre_align[,1], sep=".")); pre_align$Info; length(pre_align$Info) #change sequence IDs from only AccessionNumber to also include SpeciesNames
  cat(bold(green(paste("Part 1 of 2: align sequences for genus ", geName, sep=""))),"\n")
  
  #MSA alignment
  #store alignments per genus in alignGE at slot i (i = genus)
  pre_align.fasta <- dataframe2fas(pre_align, file = paste("GenusRaw_", geName, ".fasta", sep = "")); pre_align.fasta; class(pre_align.fasta); length(pre_align.fasta) #dataframe2fas (from seqRFLP) converts dataframe to fasta format needed in msa AND automatically outputs fasta file (editable in external software, e.g., BioEdit, Geneious) to working directory, name file = spName
  cat(italic(green(paste("Alignment in progress", sep=""))),"\n"); align_GEi <- msa(pre_align.fasta[seq(2,length(pre_align.fasta),2)], method = alig, order = "input", gapOpening = Openal, gapExtension = Epenal, maxiters = Niter, type = nu_ac); align_GEi; length(align_GEi) #do alignments/ / order = "input" keeps sequences in the order of input file
  alignGE[[i]] <- list(align_GEi); alignGE; length(alignGE) 
  #do alignments
  align <- as.data.frame(align_GEi@unmasked); dim(align); rownames(align); length(rownames(align)) #retrieve aligned sequences
  rownames(align); rownames(align) <- pre_align$Info; rownames(align) #retrieve Accession Numbers and Species names
  genus_iA <- dataframe2fas(align, file = paste("GenusAligned_", geName, ".fasta", sep = "")); length(genus_iA) #dataframe2fas (from seqRFLP) converts dataframe to fasta format needed in msa AND automatically outputs fasta file (editable in external software, e.g., BioEdit, Geneious) to working directory, name file = geName
  cat(italic(silver(paste("Alignment completed and stored in working directory",sep=""))),"\n")
  #NOTE: loop generates two files per genus, one including unaligned sequences and named = GenusRaw_GenusName.fasta, another including aligned sequences and named = GenusAligned_GenusName.fasta
  
  
  ##PART 2: DIVERGENCE ##############################
  cat(bold(green(paste("Part 2 of 2: estimate sequence divergence for genus ", geName, sep=""))),"\n")
  #create temporary fasta file containing alignment
  cat(italic(green(paste("Opening alignment", sep=""))),"\n")
  align2 <- as.data.frame(align_GEi@unmasked); align2; dim(align2) #retrieve aligned sequences again
  pre_align2 <- as.data.frame(genus_i[,c("Info", "Seq")]); pre_align; pre_align2$Info; class(pre_align2); length(pre_align2) #subsets columns 5 (Accession number) and 6 (sequences)
  rownames(align2); rownames(align2) <- paste(genus_i$Species, genus_i$Info, sep = "."); rownames(align2) #retrieve Accession Numbers
  genus_iA2 <- dataframe2fas(align2, file = paste("TempGenusAligned_", geName, ".fasta", sep = "")); genus_iA2; rownames(genus_iA2) #dataframe2fas (from seqRFLP)
  #call name of fasta file containing alignment
  file.name <- paste("TempGenusAligned_", geName, ".fasta", sep = ""); file.name
  #read alignment
  gen_align <- read.alignment(file = file.name, format = "fasta"); gen_align #where columns nb = numeric(number of sequences), nam = c(Accession Numbers), seq = list(Sequences)
  
  #estimate pairwise divergence in triangular matrix (=triangular matrix)
  gen_diver <- as.matrix(dist.alignment(gen_align, matrix = c("identity"))); gen_diver #note that row and column labels = Genus.Species.AccessionNumber
  #subset lower triangular matrix and leave off diagonal
  L.lowertriangle <- which(lower.tri(gen_diver,diag=F), arr.ind = TRUE); L.lowertriangle; dim(L.lowertriangle) #pairwise (column x row indices) combinations in triangular matrix gen_diver / for x values in the lower triangular, the number of pairwise combinations = x(x-1)/2
  cat(italic(silver(paste("+ divergence estimation in progress", sep=""))),"\n"); gen_diver <- as.matrix(dist.alignment(gen_align, matrix = c("identity"))); gen_diver #note that row and column labels = Genus.Species.AccessionNumber
  #dataframe pairwise Accession Numbers (columns 1=singleton1 & 2=singleton2) and divergence estimate (column 3=divergence)
  genF <- data.frame(    singleton1 = dimnames(gen_diver)[[2]][L.lowertriangle[,2]] ,
                         singleton2 = dimnames(gen_diver)[[1]][L.lowertriangle[,1]] ,
                         divergenceRaw = gen_diver[ L.lowertriangle ] ); genF; dim(genF) #get divergence for each pairwise combination / singleton1 and singleton2 = GenusSpecies.AccessionNumber in rows and columns of gen_diver
  
  genF$divergencePercent <- round((genF$divergenceRaw^2)*100, digits = 2); genF; names(genF); dim(genF) #calculates uncorrected divergence in a new column
  
  #############################
  #set original working directory
  setwd(wd_today_name)
  #create new directory in 1st loop iteration, then work on the created directory in subsequent loop iterations
  if (labelG2 == labelG2[1]) {newDIR <- dir.create(newDIRname2); setwd(newDIRname3)} else {setwd(newDIRname3)}
  ##OUTPUT PAIRWISE DIVERGENCES
  genF_ordered <- genF[order(genF$divergencePercent, decreasing = TRUE),]; genF_ordered #order output by decreasing divergence
  div_table <- genF_ordered
  write.table(div_table, paste("Divergence_PairwiseValues_",as.character(geName),sep=""),col.names= TRUE, row.names= TRUE,sep="\t") #save output file in working directory
  cat(italic(silver(paste("+ pairwise-sequence divergence completed and stored in working directory", sep=""))),"\n")
  #return to aligment folder
  setwd(nowDIR)
  #############################
  
  #get species names associated with respective accession number
  names(gen_align); length(gen_align) #column names
  gen_ind <- as.factor(gen_align$nam); gen_ind; class(gen_ind) #retrieve intra-generic accession numbers
  
  #replace AccessionNumbers in genF$species1 and genF$species2 by SpeciesNames
  genFbis <- data.frame(genF$singleton1, genF$singleton1, genF$singleton2, genF$singleton2, genF$divergencePercent); head(genFbis); dim(genFbis) #column-bind duplicates of AccessionNumber columns with genF
  names(genFbis) <- c("species1", "singleton1", "species2",  "singleton2", "divergencePercent"); head(genFbis); names(genFbis) #rename column headings
  species1_char <- as.character(genFbis$species1); species1_char; species1_char <- gsub("\\.","_", species1_char); species1_char; species1_char<- sub("(_[^_]+)_.*", "\\1", species1_char); species1_char
  species2_char <- as.character(genFbis$species2); species2_char; species2_char <- gsub("\\.","_", species2_char); species2_char; species2_char<- sub("(_[^_]+)_.*", "\\1", species2_char); species2_char
  genFbis$species1 <- species1_char; class(genFbis$species1) 
  genFbis$species2 <- species2_char; class(genFbis$species2) 
  head(genFbis) #this should show head of genFbis with species names in columns species1 and species2
  
  #get row indexes for pairwise comparisons between different species within genus
  numberRows <- c(1:dim(genFbis)[1]); numberRows; length(numberRows) #total number of pairwise comparisons
  rowsE <- which(genFbis$species1 == genFbis$species2); rowsE; length(rowsE) #row numbers comparing equal species
  row_i <- numberRows[-rowsE]; row_i; length(row_i) #remove row indices comparing equal species
  
  #subset divergence estimates for pairwise comparison between different species
  div_subset <- genFbis[c(row_i),]; div_subset
  
  #detects pairwise comparisons between different species BELOW/ABOVE a user-selected threshold
  cat(italic(silver(paste("+ detection of sequences below/above divergence threshold = ", div.threshold, " in progress", sep=""))),"\n")
  
  div_subsetBELOW <- div_subset[div_subset$divergencePercent <= div.threshold,]; div_subsetBELOW # BELOW OR EQUAL TO THRESHOLD <=
  dimDivBELOW <- dim(div_subsetBELOW)[1]; dimDivBELOW #number of divergent sequences 
  cat(italic(magenta(paste("# sequences with divergence <= ", div.threshold, " stored in repository divBELOW", sep=""))),"\n")
  if (dim(div_subsetBELOW)[1] == 0) {divBELOW[[i]] <- "NA"} else {divBELOW[[i]] <- div_subsetBELOW} #stores between-species comparisons below divergence threshold
  
  div_subsetABOVE <- div_subset[div_subset$divergencePercent > div.threshold,]; div_subsetABOVE # ABOVE THRESHOLD <=
  dimDivABOVE <- dim(div_subsetABOVE)[1]; dimDivABOVE #number of divergent sequences 
  cat(italic(magenta(paste("# sequences with divergence > ", div.threshold, " stored in repository divABOVE", sep=""))),"\n")
  if (dim(div_subsetABOVE)[1] == 0) {divABOVE[[i]] <- "NA"} else {divABOVE[[i]] <- div_subsetABOVE} #stores between-species comparisons below divergence threshold
  
  #remove temporary fasta file from working directory
  file.remove(file.name)
  
}
#outputs
alignGE; class(alignGE); length(alignGE) #alignments for genus with >1 seq (part1) where alignGE = divBELOW + divABOVE (see right below)
divBELOW; class(divBELOW); length(divBELOW) #alignment of species pairs (within each genus) BELOW divergence threshold (part2)
divABOVE; class(divABOVE); length(divABOVE) #alignment of species pairs (within each genus) ABOVE divergence threshold (part2)
cat(bold(blue(paste("Intrageneric alignments completed and stored in folder = ", newDIRname, " within your working directory", sep=""))),"\n"); cat(bold(blue(paste("Intrageneric divergences completed and stored in folder = ", newDIRname2, " within your working directory", sep=""))),"\n")


######return to initial working directory
setwd(wd_today_name)
cat(bold(blue(paste("Your working directory is back to ", wd_today_name, sep=""))))

##Alignment of species pairs (within each genus) BELOW divergence threshold 
#remove NULL and NA elements (genus < 1 species)
cat(bold(blue(paste("For GENUS BELOW divergence < ", div.threshold, ", subset only interspecific comparisons for inspection", sep=""))))
divBELOWn <- Filter(Negate(is.null),divBELOW); length(divBELOW); length(divBELOWn) #remove null elements from list
#dataframe divBELOWn /  Note that NA elements in a list are coerced to NA rows for genus < 1 species, so options(warn=-1) avoids error messages for those, and (warn=0) gets error messaging back
options(warn=-1)
divBELOWdf <- data.frame(list.rbind(divBELOWn))
options(warn=0)
#remove NA rows
divBELOWna <- divBELOWdf[!is.na(divBELOWdf$singleton1),]; table(divBELOWdf$species1); table(divBELOWna$species1); dim(divBELOWdf); dim(divBELOWna)
#subset only pairwise comparisons between different species
indexBELOW <- which(divBELOWna$species1 != divBELOWna$species2) #target rows
divBELOWsp <- divBELOWna[indexBELOW,]; dim(divBELOWsp) #subset target rows
#subset comparisons with divergence <= 3
divBELOW3 <- divBELOWsp[divBELOWsp$divergencePercent <= 3, ]; dim(divBELOW3)
cat(bold(blue(paste("There are ", dim(divBELOW3)[1], " between-species pairwise-sequence comparisons with divergence <= ", div.threshold, " - see species and comparisons involved in dataframe = divBELOW3", sep=""))))

##Alignment of species pairs (within each genus) ABOVE divergence threshold 
#remove NULL and NA elements (genus < 1 species)
cat(bold(blue(paste("For GENUS ABOVE divergence > ", div.threshold, ", subset only intraspecific comparisons for inspection", sep=""))))
divABOVEn <- Filter(Negate(is.null),divABOVE); length(divABOVE); length(divABOVEn) #remove null elements from list
#dataframe divABOVEn / Note that NA elements in a list are coerced to NA rows for genus < 1 species, so options(warn=-1) avoids error messages for those, and (warn=0) gets error messaging back
options(warn=-1)
divABOVEdf <- data.frame(list.rbind(divABOVEn))
options(warn=0)
#remove NA rows
divABOVEna <- divABOVEdf[!is.na(divABOVEdf$singleton1),]; table(divABOVEdf$species1); table(divABOVEna$species1); dim(divABOVEdf); dim(divABOVEna)
#subset only pairwise comparisons between sequences of the same species
speNam1 <- gsub("_.*","",divABOVEna$species1); length(speNam1) #cut species name from singleton 1
speNam2 <- gsub("_.*","",divABOVEna$species2); length(speNam2) #cut species name from singleton 2
indexABOVE <- which(speNam1 == speNam2);length(indexABOVE) #target rows
divABOVEsp <- divABOVEna[indexABOVE,]; dim(divABOVEsp) #subset target rows
#subset comparisons with divergence > 3
divABOVE3 <- divABOVEsp[divABOVEsp$divergencePercent > 3, ]; dim(divABOVE3)
cat(bold(blue(paste("There are ", dim(divABOVE3)[1], " within-species pairwise-sequence comparisons with divergence > ", div.threshold, " - see species and comparisons involved in dataframe = divABOVE3", sep=""))))

#######################################################
#VISUALIZATION OF SEQUENCES FROM GENUS WITH > 1 SPECIES
#table showing number of species per genus
scoreGS 
#vector extracting genus with more than 1 species from dataframe scoreGS
table_scoreGS <- scoreGS[scoreGS >1]; table_scoreGS
#vector extracting names of genus > 1 species
ge1plusnames <- c(names(table_scoreGS)); ge1plusnames
#dataframe extracting sequences for genus > 1 species / if many, this file will be relatively large
speciesG[speciesG$nameGenus %in% ge1plusnames,] 
#######################################################

#######################################################
#VISUALIZATION OF SEQUENCES BELOW DIVERGENCE THRESHOLD
#function removing null elements from list
delete.NULLs  <-  function (x.list) { x.list[unlist(lapply(x.list, length) != 0)] } 
#application of function to visualize genus including cross-species sequences below divergence threshold
div_alignGE_Null <- delete.NULLs(divBELOW); div_alignGE_Null
#NOTE: list element will have 0 rows if genus contains no divergence below threshold
#######################################################


###################################################################################################################################
#STEP 8: Identify information per base pair for database alignment
###################################################################################################################################
#set working directory
setwd(wd_today_name); wd_today <- setwd(wd_today_name) #/path for Mac
cat(bold(blue(paste("Your working directory is ", wd_today_name, sep=""))))

#load data / Thijs: provide users here an explanation of what the file "name of file.txt" should contain
cons0 <- read.table("name of file.txt", sep="\t", dec=".", header = T);
summary(cons0); dim(cons0); names(cons0)
cons0$speciesName <- paste(cons0$genus, cons0$species, sep ="_") #create column vector with species name
summary(cons0); dim(cons0); names(cons0)

#reorder by speciesName
cons <- cons0[order(cons0$speciesName),] #order rows by species name in alphabetical order
head(cons$speciesName); tail(cons$speciesName)
taxIndex <- table(cons$speciesName) #number of sequences per species in alphabetical order
speciesNumber <- length(taxIndex); speciesNumber # number of species
sum(table(cons$speciesName)); dim(cons) #total number of sequences (= dim(cons)[1])

#splitting base strings
class(as.character(cons$sequence))
consC <- as.character(cons$sequence); class(consC) #convert each sequence string from factor to character
consS <- strsplit(consC,""); class(consC); length(consS); length(consS[[1]]) #split each sequence into successive, individual bases
baseN <- length(consS[[1]]); baseN #number of base position
consD <- t(data.frame(consS)); dim(consD) #convert sequences to data frame
rownames(consD) <- as.character(cons$speciesName); colnames(consD) <- paste("pos", c(1:baseN), sep=""); dim(consD) #renames rows and columns
head(consD); colnames(consD)


################# MISSING VALUES PER POSITION
#count non-missing values for (example) position 1
consD[,1]
length(which(consD[,1] != "-"))

#count non-missing values for position 1 to 1577
countFinal <- apply(consD, 2, function (x) length(which(x != "-")))

#plot results
par(mar=c(4.5,5,3,0.5))
plot(countFinal, ylab = "Number of sequences without missing value", xlab = "Base position (1 to ####) in cytb gene", cex.lab = 2,
     main = "#### amphibian sequences", cex.main = 3)
abline(v=seq(0,1600,100), col = "lightgrey", lty ="dashed")


################# MISSING VALUES PER POSITION AND SPECIES
#count nonmissing values per species across positions
repoSpe <- matrix(NA, nrow = speciesNumber, ncol = dim(consD)[2]); dim(repoSpe) #repository with nrow = number species and ncol = number of positions
for (j in c(1:dim(consD)[2])) {
  naNumber <- tapply(consD[,j], rownames(consD), function (x) length(which(x != "-"))); length(naNumber) #count number of nonmissing values
  repoSpe[,j] <- naNumber
}
rownames(repoSpe) <- names(taxIndex); colnames(repoSpe) <- colnames(consD)

#count number of species with nonmissing values per position
countFinal2 <- apply(repoSpe, 2, function (x) length(which(x != 0)))

#plot results
par(mar=c(4.5,5,3,0.5))
plot(countFinal2, ylab = "Number of species without missing value", xlab = "Base position (1 to ####) in cytb gene", cex.lab = 2,
     main = "#### amphibian species", cex.main = 3)
abline(v=seq(0,1600,100), col = "lightgrey", lty ="dashed")


################# MISSING VALUES PER POSITION VERSUS MISSING VALUES PER POSITION AND SPECIES
length(countFinal); length(countFinal2) 
par(mar=c(4.5,5,3,0.5))
plot(countFinal, countFinal2, cex.lab = 2,
     ylab = "Number of species without missing value (N = ####)", xlab = "Number of sequences without missing values (N = ####)",
     main = "#### positions in cytb gene", cex.main = 3)

###############################################################################################################################################################################################################################################
#SCRIPT END
###############################################################################################################################################################################################################################################